<?php
/**
 * $Horde: klutz/backend.php,v 1.27.2.6 2005/01/03 11:25:56 jan Exp $
 *
 * Copyright 2002-2005 Marcus I. Ryan <marcus@riboflavin.net>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('KLUTZ_BASE', dirname(__FILE__));

$compression_control = 'none';
$self_contained_auth = true;
require_once KLUTZ_BASE . '/lib/base.php';

/* Are we going to use Horde auth or HTTP auth? */
if (isset($conf['auth']['http']) && is_array($conf['auth']['http'])) {
    if (empty($_SERVER['PHP_AUTH_USER']) ||
        ($_SERVER['PHP_AUTH_USER'] !== $conf['auth']['http']['username']) &&
        empty($_SERVER['PHP_AUTH_PW']) ||
        empty($conf['auth']['http']['password']) ||
        ($_SERVER['PHP_AUTH_PW'] !== $conf['auth']['http']['password']) ||
        !in_array($_SERVER['REMOTE_ADDR'], $conf['auth']['http']['ips'])) {

        header('WWW-Authenticate: Basic realm="Klutz"');
        header('HTTP/1.0 401 Unauthorized');
        exit('forbidden');
    }
} else {
    /* Check that a Horde admin is logged in. */
    if (!in_array(Auth::getAuth(), $conf['auth']['admins'])) {
        exit('forbidden.');
    }
}

// Don't do anything if the backend is none.
if ($conf['storage']['driver'] == 'none') {
    $notification->push(_("You must define a storage backend to use the administrative interface."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('comics.php'));
    $stderr = fopen('php://stderr', 'w');
    fwrite($stderr, _("You must define a storage backend to use the administrative interface."));
    exit(1);
}

// Proceed once we're sure we're authorized.
if (!isset($mode)) {
    $mode = Horde::getFormData('mode', array('menu'));
}
if (!is_array($mode)) {
    $mode = array($mode);
}

$time = mktime(0, 0, 0);
$t = getdate($time);
if (array_key_exists('backend', $conf) && array_key_exists('daystokeep', $conf['backend'])) {
    $daystokeep = $conf['backend']['daystokeep'];
    $oldest = mktime(0, 0, 0, $t['mon'], $t['mday'] - $daystokeep, $t['year']);
} else {
    $daystokeep = 90;
    $oldest = null;
}
$expired = mktime(0, 0, 0, $t['mon'], $t['mday'] - ($daystokeep + 1), $t['year']);

if (in_array('sums', $mode)) {
    $storage->rebuildSums();
    if (in_array('menu', $mode)) {
        $notification->push(_("The unique identifiers table has been rebuilt."));
    } else {
        echo _("The unique identifiers table has been rebuilt.");
    }
}

if (count($mode) == 0 || in_array('menu', $mode)) {
    $fetch_date_select = '<select name="date">';
    $fetch_date_select .= '<option value="all">' . _("All Dates") . '</option>';
    foreach (array_reverse(Klutz_Driver::listDates($time, $oldest)) as $date) {
        if ($date == $time) {
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        $fetch_date_select .= '<option value="' . $date . '"' . $selected . '">';
        $fetch_date_select .= strftime('%B %d, %Y', $date);
        $fetch_date_select .= '</option>';
    }
    $fetch_date_select .= '</select>';

    $delete_date_select = '<select name="date">';
    foreach ($storage->listDates($time, 0, $time) as $date) {
        if ($date == $oldest) {
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        $delete_date_select .= '<option value="' . $date . '"' . $selected . '">';
        $delete_date_select .= strftime("%B %d, %Y", $date);
        $delete_date_select .= '</option>';
    }
    $delete_date_select .= '</select>';

    $comic_select = '<select name="index">';
    $comic_select .= '<option value="all" selected="selected">' .
        _("All Comics") . '</option>';
    foreach ($klutz->listEnabled() as $index) {
        $comic_select .= "<option value=\"$index\">" .
            $klutz->getProperty($index, 'name') .
            '</option>';
    }
    $comic_select .= '</select>';

    $sums_url = Horde::applicationUrl('backend.php');
    $sums_url = Horde::addParameter($sums_url, 'mode[]=sums');
    $sums_url = Horde::addParameter($sums_url, 'mode[]=menu');
    
    $title = _("Comics Update");
    include KLUTZ_TEMPLATES . '/common-header.inc';
    include KLUTZ_BASE . '/menu.php';
    $notification->notify();
    include KLUTZ_TEMPLATES . '/backend/backend.inc';
    include KLUTZ_TEMPLATES . '/common-footer.inc';
    exit;
}

if (in_array('fetch', $mode)) {
    $date = Horde::getFormData('date', mktime(0, 0, 0));
    $index = Horde::getFormData('index', 'all');
    $overwrite = Horde::getFormData('overwrite', 'false');
    $nounique = Horde::getFormData('nounique', 'false');

    if (strtolower($overwrite) == 'true') {
        $overwrite = true;
    } else {
        $overwrite = false;
    }
    if (strtolower($nounique) == 'true') {
        $nounique = true;
    } else {
        $nounique = false;
    }

    if (strtolower($date) == 'all') {
        $dates = array_reverse(Klutz_Driver::listDates($time, $oldest));
    } else {
        // make sure the time for $date is midnight
        $d = getdate($date);
        $date = mktime(0, 0, 0, $d['mon'], $d['mday'], $d['year']);
        $dates = array($date);
    }

    foreach ($dates as $date) {
        // is $date today?
        $today = false;
        if ($date != "all" && $date == mktime(0, 0, 0)) {
            $today = true;
        }

        if (strtolower($index) == 'all') {
            $comics = $klutz->listEnabled(null, $date);
        } else {
            $comics = $klutz->listEnabled(array($index), $date);
        }

        foreach ($comics as $comic) {
            // If this comic isn't available all days, nohistory gets
            // ugly. SOMEONE PLEASE MAKE IT BETTER ;)
            $name = $klutz->getProperty($comic, 'name');
            $days = $klutz->getProperty($comic, 'days');
            if ($days != 'random' && count($days) < 7) {
                $day = mktime(0, 0, 0);
                while (!in_array(strtolower(date('D', $day)),$days)) {
                    $d = getdate($day);
                    $day = mktime(0, 0, 0, $d['mon'], $d['mday'] - 1, $d['year']);
                }
                if ($day == $date) {
                    $today = true;
                }
            }

            if (!$today && $klutz->getProperty($comic, 'nohistory')) {
                printf(_("Skipping %s for %s- no historical fetching"),
                       $name, strftime("%B %d, %Y", $date));
            } elseif (!$storage->imageExists($comic, $date) || $overwrite) {
                printf(_("Fetching %s for date %s..."),
                       $name, strftime('%B %d, %Y', $date));
                flush();

                $c = $klutz->comicObject($comic);
                $image = $c->fetchImage($date);
                if (!is_object($image)) {
                    echo _("The image doesn't exist or can't be retrieved");
                } elseif (!$nounique && (method_exists($storage, "isUnique") &&
                          !$storage->isUnique($image))) {
                    echo _("This image appears to be a repeat");
                    echo " (" . md5($image->data) . ")";
                } elseif ($storage->storeImage($comic, $image, $date)) {
                    echo _("Done");
                } else {
                    echo _("Error storing the image");
                }
            } else {
                printf(_("Skipping %s for %s"),
                       $name, strftime('%B %d, %Y', $date));
            }
            echo "<br />\n";
        }
    }

    // Save the updated sums if necessary
    if (method_exists($storage, 'saveSums')) {
        $storage->saveSums();
    }
}

if (in_array('delete', $mode)) {
    $date = Horde::getFormData('date', $expired);
    $index = Horde::getFormData('index', 'all');
    $timeframe = Horde::getFormData('timeframe', 'older');

    // make sure the time for $date is midnight
    $d = getdate($date);
    $date = mktime(0, 0, 0, $d['mon'], $d['mday'], $d['year']);

    $success = _("Successfully removed %s for %s<br />\n");
    $notfound = _("Could not find %s for %s<br />\n");
    $error = _("An error occurred removing %s for %s<br />\n");

    // Figure out what dates we need to work on
    if ($timeframe == 'date') {
        $dates = array($date);
    } elseif ($timeframe == 'older') {
        $dates = $storage->listDates($date, 0, $date);
    } elseif ($timeframe == 'newer') {
        $dates = $storage->listDates($date, $date, mktime(0, 0, 0));
    } else {
        $dates = array();
    }

    if ($index == 'all') {
        if (count($dates) == 0) {
            printf($notfound, _("any comics"), _("any dates"));
        }
        foreach ($dates as $date) {
            if ($storage->removeDate($date)) {
                printf($success, _('all comics'), strftime('%B %d, %Y', $date));
            } else {
                printf($error, _('all comics'), strftime('%B %d, %Y', $date));
            }
        }
    } else {
        $name = $klutz->getProperty($index, 'name');
        $days = $klutz->getProperty($index, 'days');

        foreach ($dates as $date) {
            if ($days == 'random' || in_array(strtolower(date('D', $date)), $days)) {
                if (!$storage->imageExists($index, $date)) {
                    printf($notfound, $index, strftime('%B %d, %Y', $date));
                } else {
                    if ($storage->removeImage($index, $date)) {
                        printf($success, $name, strftime('%B %d, %Y', $date));
                    } else {
                        printf($error, $name, strftime('%B %d, %Y', $date));
                    }
                }
            }
        }
    }

    // Save the updated sums if necessary.
    if (method_exists($storage, 'saveSums')) {
        $storage->saveSums();
    }
}
